<?php
/**
 * Config class
 *
 * @since 1.0
 */
namespace dodebug;

defined( 'WPINC' ) || exit;

class Conf extends Instance
{
	protected static $_instance;

	private $_options = array();

	protected static $_default_options = array(
		'_ver'				=> '',
		'debug'				=> true,
		'log_cookie'		=> false,
		'log_size'			=> 3,
		'adv'				=> false,
		'uri_inc'			=> array(),
		'uri_exc'			=> array(),
		'log_filter'		=> false,
		'filter_exc'		=> array( 'gettext', 'gettext_with_context', 'get_the_terms', 'get_term', 'attribute_escape', 'clean_url' ),
		'filter_partial_exc'=> array( 'i18n', 'locale', 'settings', 'option' ),
		'admin_ip'			=> array(),
	);

	protected function __construct()
	{
	}

	/**
	 * Init config
	 *
	 * @since  1.0
	 * @access public
	 */
	public function init()
	{
		// Load all options
		$options = array();
		foreach ( self::$_default_options as $k => $v ) {
			$options[ $k ] = $this->_get_option( $k, $v );
		}

		$this->_options = $options;

		// Update options if not exists
		! defined( 'DODEBUG_CUR_V' ) && define( 'DODEBUG_CUR_V', $this->_options[ '_ver' ] ) ;

		if ( ! DODEBUG_CUR_V || DODEBUG_CUR_V != Core::VER ) {
			if ( ! DODEBUG_CUR_V ) {
			}
			else {
				// DB update
				Data::get_instance()->conf_upgrade();
			}

			foreach ( self::$_default_options as $k => $v ) {
				self::add( $k, $v );
			}

			self::update( '_ver', Core::VER );
		}
	}

	/**
	 * Get one current option
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function val( $id )
	{
		$instance = self::get_instance();
		if ( isset( $instance->_options[ $id ] ) ) {
			return $instance->_options[ $id ];
		}

		return null;
	}

	/**
	 * Get all options
	 *
	 * @since  1.1
	 * @access private
	 */
	public function get_options()
	{
		return $this->_options;
	}

	/**
	 * Add one option
	 *
	 * @since  1.4.1
	 * @access public
	 */
	public static function add( $id, $v )
	{
		add_option( 'dodebug.' . $id, $v );
	}

	/**
	 * Delete one option
	 *
	 * @since  1.4.1
	 * @access public
	 */
	public static function delete( $id )
	{
		delete_option( 'dodebug.' . $id );
	}

	/**
	 * Get option from DB
	 *
	 * @since  1.0
	 * @access private
	 */
	private function _get_option( $id, $default_v = false )
	{
		return get_option( 'dodebug.' . $id, $default_v );
	}

	/**
	 * Update option of dodebug
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function update( $id, $data )
	{
		if ( ! array_key_exists( $id, self::$_default_options ) ) {
			return;
		}

		// typecast
		$default_v = self::$_default_options[ $id ];
		if ( is_bool( $default_v ) ) {
			$data = (bool) $data;
		}
		elseif ( is_array( $default_v ) ) {
			if ( ! is_array( $data ) ) {
				$data = explode( "\n", $data );
				$data = array_filter( $data );
			}
		}
		elseif ( ! is_string( $default_v ) ) {
			$data = (int) $data;
		}

		update_option( 'dodebug.' . $id, $data );

		// Special handler for removal of log files
		if ( $id == 'debug' ) {
			if ( ! $data && self::get_instance()->_options[ $id ] ) {
				Log::get_instance()->clear_log( true );
			}
		}

		// Change current setting
		self::get_instance()->_options[ $id ] = $data;

	}

}
