<?php
/**
 * Router class
 *
 * @since 1.0
 */
namespace dodebug;

defined( 'WPINC' ) || exit;

class Router extends Instance
{
	protected static $_instance;

	const NONCE = 'dodebug_nonce';
	const ACTION = 'dodebug_action';
	const TYPE = 'dodebug_type';
	const I = 'dodebug_i';

	private static $_action;

	/**
	 * Init
	 */
	public function init()
	{
		add_action( 'init', array( $this, 'after_user_init' ) );
	}

	/**
	 * Proceed actions
	 *
	 * @since  1.0
	 */
	public function after_user_init()
	{
		$action = Router::get_action();
		switch ( $action ) {
			default:
				break;
		}

		if ( $action ) {
			self::redirect();
		}
	}

	/**
	 * Redirect page and drop self params
	 *
	 * @since  1.0
	 */
	public static function redirect( $url = false )
	{
		global $pagenow;
		$qs = '';
		if ( ! $url ) {
			if ( ! empty( $_GET ) ) {
				if ( isset( $_GET[ self::ACTION ] ) ) {
					unset( $_GET[ self::ACTION ] );
				}
				if ( isset( $_GET[ self::NONCE ] ) ) {
					unset( $_GET[ self::NONCE ] );
				}
				if ( isset( $_GET[ self::TYPE ] ) ) {
					unset( $_GET[ self::TYPE ] );
				}
				if ( isset( $_GET[ self::I ] ) ) {
					unset( $_GET[ self::I ] );
				}
				if ( ! empty( $_GET ) ) {
					$qs = '?' . http_build_query( $_GET );
				}
			}
			if ( is_network_admin() ) {
				$url = network_admin_url( $pagenow . $qs );
			}
			else {
				$url = admin_url( $pagenow . $qs );
			}
		}

		wp_redirect( $url );
		exit();
	}

	/**
	 * Parse action
	 *
	 * @since  1.0
	 */
	public static function get_action()
	{
		if ( ! isset( self::$_action ) ) {
			self::$_action = false;
			self::get_instance()->verify_action();
			if ( self::$_action ) {
				defined( 'debug' ) && debug( '[Router] dodebug action verified: ' . var_export( self::$_action, true ) );
			}

		}
		return self::$_action;
	}

	/**
	 * Verify action
	 *
	 * @since  1.0
	 */
	private function verify_action()
	{
		if ( empty( $_REQUEST[ Router::ACTION ] ) ) {
			defined( 'debug' ) && debug2( '[Router] do_debug action bypassed empty' );
			return;
		}

		$action = $_REQUEST[ Router::ACTION ];

		if ( ! $this->verify_nonce( $action ) ) {
			return;
		}

		$_can_option = current_user_can( 'manage_options' );

		switch ( $action ) {
			default:
				defined( 'debug' ) && debug( '[Router] do_debug match falied: ' . $action );
				return;
		}

	}

	/**
	 * Verify nonce
	 *
	 * @since  1.0
	 */
	private function verify_nonce( $action )
	{
		if ( ! isset( $_REQUEST[ Router::NONCE ] ) || ! wp_verify_nonce( $_REQUEST[ Router::NONCE ], $action ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Get type value
	 *
	 * @since 1.0
	 * @access public
	 */
	public static function verify_type()
	{
		if ( empty( $_REQUEST[ self::TYPE ] ) ) {
			defined( 'debug' ) && debug( '[Router] no type', 2 ) ;
			return false ;
		}

		defined( 'debug' ) && debug( '[Router] parsed type: ' . $_REQUEST[ self::TYPE ], 2 ) ;

		return $_REQUEST[ self::TYPE ] ;
	}

}